# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

densne_cpp <- function(X, no_dims, perplexity, theta, verbose, max_iter, Y_in, init, stop_lying_iter, mom_switch_iter, momentum, final_momentum, eta, exaggeration_factor, dens_frac, dens_lambda, final_dens, num_threads) {
    .Call('_densvis_densne_cpp', PACKAGE = 'densvis', X, no_dims, perplexity, theta, verbose, max_iter, Y_in, init, stop_lying_iter, mom_switch_iter, momentum, final_momentum, eta, exaggeration_factor, dens_frac, dens_lambda, final_dens, num_threads)
}

